@@ -23,7 +23,8 @@ from optparse import OptionParser
 from mercurial import hg, ui as uimod
 from mercurial.error import RepoError
 
-from hgviewlib.util import find_repository, rootpath
+from hgviewlib import __pkginfo__
+from hgviewlib.util import find_repository, rootpath, build_repo
 from hgviewlib.config import HgConfig
 
 class NullRepo(object):
@@ -35,7 +36,7 @@ class Viewer(object):
     """Base viewer class interface."""
     def __init__(self, *args, **kwargs):
         raise NotImplementedError(
-            'This feature has not yet been implemented. Comming soon ...')
+            'This feature has not yet been implemented. Coming soon ...')
 
 class FileViewer(Viewer):
     """Single file revision graph viewer."""
@@ -58,10 +59,10 @@ class ManifestViewer(Viewer):
         super(ManifestViewer, self).__init__(**kwargs)
 
 class ApplicationError(ValueError):
-    """Exception that may occures while lunching the application"""
+    """Exception that may occur while launching the application"""
 
 class HgViewApplication(object):
-    # class that must be instancied
+    # class that must be instantiated
     FileViewer = FileViewer
     FileDiffViewer = FileDiffViewer
     HgRepoViewer = HgRepoViewer
@@ -178,7 +179,7 @@ def start(repo, opts, args, fnerror):
 
 def main():
     """
-    Main application acces point.
+    Main application entry point.
     """
 
     usage = '''%prog [options] [filename]
@@ -194,7 +195,7 @@ def main():
       revision.
     '''
 
-    parser = OptionParser(usage)
+    parser = OptionParser(usage, version=__pkginfo__.version)
     parser.add_option('-I', '--interface', dest='interface',
                       help=('which GUI interface to use (among "qt", "raw"'
                              ' and "curses")'),
@@ -218,7 +219,7 @@ def main():
     try:
         if repopath:
             u = uimod.ui()
-            repo = hg.repository(u, repopath)
+            repo = build_repo(u, repopath)
         else:
             repo = NullRepo()
         try:
