@@ -48,16 +48,18 @@ class HgConfig(object):
         self.section = section
         self._cache = {}
 
-    def _fromconfig(self, name, default):
+    def _fromconfig(self, name, default, configmethod='config'):
         '''allow per-interface configuration.
         look for ``interface.config`` then for ``config`` if the first were not
         found'''
-        out = self.ui.config(self.section,
-                             '.'.join((self.ui.opts.interface, name)),
-                             None)
+        getconfig = getattr(self.ui, configmethod)
+        out = getconfig(self.section,
+                        '.'.join((self.ui.opts.interface, name)),
+                        None)
         if out is not None:
             return out
-        return self.ui.config(self.section, name, default)
+        return getconfig(self.section, name, default)
+
 
     @cached
     def getFancyReplace(self):
@@ -209,7 +211,7 @@ class HgConfig(object):
     def getChangelogColumns(self, default=None):
         """
         changelogcolumns: ordered list of displayed columns in changelog views;
-                    defaults to ID, Branch, Log, Author, Date, Tags
+                    defaults to ID, Branch, Log, Author, Date
         """
         cols = self._fromconfig('changelogcolumns', default)
         if cols is None:
@@ -298,14 +300,16 @@ class HgConfig(object):
         """
         contentatstartup: show the content of changeset at startup (bottom part)
         """
-        return bool(self._fromconfig('contentatstartup', default))
+        return bool(self._fromconfig('contentatstartup', default,
+                                     configmethod='configbool'))
 
     @cached
     def getShowHidden(self, default=False):
         """
         showhidden: show hidden changeset at startup
         """
-        return bool(self._fromconfig('showhidden', default))
+        return bool(self._fromconfig('showhidden', default,
+                                     configmethod='configbool'))
 
     @cached
     def getInterface(self, default=None):
@@ -320,14 +324,16 @@ class HgConfig(object):
         nonpublicontop: display non public changesets on top of the graph log
                         (disabled with *show hidden*)
         """
-        return bool(self._fromconfig('nonpublicontop', default))
+        return bool(self._fromconfig('nonpublicontop', default,
+                                     configmethod='configbool'))
 
     @cached
     def getShowObsolete(self, default=True):
         """
         showobsolete: display obsolete relations
         """
-        return bool(self._fromconfig('showobsolete', default))
+        return bool(self._fromconfig('showobsolete', default,
+                                     configmethod='configbool'))
 
     @cached
     def getExportTemplate(self):
