@@ -25,6 +25,7 @@ import difflib
 from itertools import chain, count, imap
 from time import strftime, localtime
 from functools import partial
+import collections
 
 from mercurial.node import nullrev
 from mercurial import patch, util, match, error, hg
@@ -76,21 +77,46 @@ def diff(repo, ctx1, ctx2=None, files=None):
     return tounicode(diffdata)
 
 
-def __get_parents(repo, rev, branch=None):
+def __get_parents(repo, rev, subset, cache):
     """
-    Return non-null parents of `rev`. If branch is given, only return
-    parents that belongs to names branch `branch` (beware that this is
-    much slower).
+    Return non-null parents of `rev`. Only revision in subset are taken in
+    account.
     """
-    if not branch:
-        if rev is None:
-            return [x.rev() for x in repo.changectx(None).parents() if x]
-        return [x for x in repo.changelog.parentrevs(rev) if x != nullrev]
-    if rev is None:
-        return [x.rev() for x in repo.changectx(None).parents() \
-                if x and repo.changectx(rev).branch() == branch]
-    return [x for x in repo.changelog.parentrevs(rev) \
-            if (x != nullrev and repo.changectx(rev).branch() == branch)]
+    # we need to manage our own stack or we overflow python one.
+    toproceed = collections.deque()
+    if rev not in cache:
+        toproceed.append(rev)
+    else:
+        final = cache[rev]
+    while toproceed:
+        current = toproceed.pop()
+        if current is None:
+            parents = [x.rev() for x in repo.changectx(None).parents()
+                       if x]
+        else:
+            parents = [x for x in repo.changelog.parentrevs(current)
+                       if x != nullrev]
+        final = []
+        added = set()
+        for p in parents:
+            if p in subset:
+                final.append(p)
+                added.add(p)
+            else:
+                if p not in cache:
+                    # stack the parent and jump to next iteration
+                    toproceed.append(current)
+                    toproceed.append(p)
+                    break
+                else:
+                    for p in cache[p]:
+                        if p not in added:
+                            final.append(p)
+                            added.add(p)
+        else:
+            # all parents in cache register the new one
+            cache[current] = final
+    return final
 
 def getlog(model, ctx, gnode):
     if ctx.rev() is not None:
@@ -101,7 +127,7 @@ def getlog(model, ctx, gnode):
         msg = u"WORKING DIRECTORY (locally modified)"
     return msg
 
-def gettags(model, ctx, gnode):
+def gettags(model, ctx, gnode=None):
     if ctx.rev() is None:
         return u""
     mqtags = ['qbase', 'qtip', 'qparent']
@@ -154,7 +180,14 @@ def _rev_order(phaserevs, rev):
     return (phases.public != phaserevs[rev], rev)
 
 def _build_filter(start_rev, branch, follow, closed):
-    """process filter rules and returns maching revisions as list"""
+    """process filter rules and returns maching revisions as list
+
+    If start_rev is None, started from working directory node
+
+    If follow is True, only generated the subtree from the start_rev head.
+
+    If branch is set, only generated the subtree for the given named branch.
+    """
     # selected range
     revset_args = []
     if (start_rev is None):
@@ -204,8 +237,7 @@ def _revset_revs(repo, revset, revset_args=(), view='visible'):
             revs = [r for r in revs if r not in excluded]
         return revs
 
-def revision_grapher(repo, start_rev=None, branch=None, follow=False,
-                     show_hidden=False, reorder=False, closed=False,
+def revision_grapher(repo, revset_filter, start_rev=None, show_hidden=False, reorder=False,
                      show_obsolete=False):
     """incremental revision grapher
 
@@ -220,16 +252,7 @@ def revision_grapher(repo, start_rev=None, branch=None, follow=False,
         the current row and the next row
       - parent revisions of current revision
 
-    If start_rev is None, started from working directory node
-
-    If follow is True, only generated the subtree from the start_rev head.
-
-    If branch is set, only generated the subtree for the given named branch.
-
     """
-    # follow is disabled when start_rev is None (== not revision specified)
-    follow = start_rev is not None and follow # disable follow is start_rev is None
-
     # special handling of mq patches
     if show_hidden and start_rev == None and hasattr(repo, 'mq'):
         series = list(reversed(repo.mq.series))
@@ -237,12 +260,8 @@ def revision_grapher(repo, start_rev=None, branch=None, follow=False,
             if not repo.mq.isapplied(patchname):
                 yield (patchname, 0, 0, [(0, 0 ,0, False)], [])
 
-    # computed hidden revs
-    excluded = () if show_hidden else hiddenrevs(repo)
-
-    revset, args = _build_filter(start_rev, branch, follow, closed)
     view = None if show_hidden else 'visible'
-    included = _revset_revs(repo, revset, args, view=view)
+    included = _revset_revs(repo, *revset_filter, view=view)
     if getattr(included, 'append', None) is None:
         # _revset_revs returned a smartset (new in Mercurial 3.0)
         # translate it back to list for now
@@ -259,6 +278,12 @@ def revision_grapher(repo, start_rev=None, branch=None, follow=False,
         included.sort(key=keyfunc)
 
     included.reverse()
+    inc_set = set(included)
+    # <excluded> necessary for first_known_precursors
+    # the second user of first_known_precursors can't really
+    # reverse its logic.
+    excluded = set(repo) - inc_set
+    parentscache = {}
 
     # all known revs for this line. This is used to compute column index
     # it's combined with next_revs to compute how we must draw lines
@@ -268,6 +293,8 @@ def revision_grapher(repo, start_rev=None, branch=None, follow=False,
                     #            False for weak one (obsolete)
     rev_color = {}
     free_color = count(0)
+    parent_func = partial(__get_parents, repo=repo, subset=inc_set,
+                          cache=parentscache)
     for curr_rev in included:
         # Compute revs and next_revs.
         if curr_rev not in revs: # rev not ancestor of already processed node
@@ -282,7 +309,7 @@ def revision_grapher(repo, start_rev=None, branch=None, follow=False,
         next_levels = levels[:]
 
         # Add parents to next_revs.
-        parents = [(p, True) for p in __get_parents(repo, curr_rev, branch)]
+        parents = [(p, True) for p in parent_func(rev=curr_rev)]
         if show_obsolete:
             ctx = repo[curr_rev]
             for prec in first_known_precursors(ctx, excluded):
@@ -676,11 +703,10 @@ class HgRepoListWalker(object):
         for branch_name in chain(['default', 'stable'],
                                  sorted(allbranches(repo, True))):
             self.namedbranch_color(branch_name)
-        grapher = revision_grapher(self.repo, start_rev=fromhead,
-                                   follow=follow, branch=branch,
+        revset_filter = _build_filter(fromhead, branch, follow, closed)
+        grapher = revision_grapher(self.repo, revset_filter, start_rev=fromhead,
                                    show_hidden=self.show_hidden,
                                    reorder=self.reorder_changesets,
-                                   closed=closed,
                                    show_obsolete=self.show_obsolete)
 
         self.graph = Graph(self.repo, grapher, self.max_file_size)
