@@ -15,13 +15,18 @@
 """
 Qt4 high level widgets for hg repo changelogs and filelogs
 """
+import os
 import difflib
 from itertools import imap
+import tempfile
+
+
 
 try:
-    from mercurial.error import LookupError as HgLookupError
+    from mercurial.error import LookupError, ManifestLookupError
 except ImportError:
-    from mercurial.revlog import LookupError as HgLookupError
+    # ManifestLookupError is missing in older versions
+    from mercurial.revlog import LookupError, LookupError as ManifestLookupError
 
 from PyQt4 import QtCore, QtGui, Qsci
 from PyQt4.QtCore import Qt, pyqtSignal
@@ -29,170 +34,81 @@ from PyQt4.QtCore import Qt, pyqtSignal
 from hgviewlib.util import exec_flag_changed, isbfile, bfilepath, tounicode
 from hgviewlib.config import HgConfig
 
-from hgviewlib.qt4 import icon as geticon
-from hgviewlib.qt4.hgfiledialog import FileViewer, FileDiffViewer
-from hgviewlib.qt4.lexers import get_lexer
+from hgviewlib.qt4.mixins import ActionsMixin
+from hgviewlib.qt4.config import get_font
 from hgviewlib.qt4.blockmatcher import BlockList
-
-qsci = Qsci.QsciScintilla
-
-class Annotator(qsci):
-    # we use a QScintilla for the annotator cause it makes
-    # it much easier to keep the text area and the annotator sync
-    # (same font rendering etc). However, it have the drawback of making much
-    # more difficult to implement things like QTextBrowser.anchorClicked, which
-    # would have been nice to directly go to the annotated revision...
-    def __init__(self, textarea, parent=None):
-        qsci.__init__(self, parent)
-
-        self.setFrameStyle(0)
-        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        self.setReadOnly(True)
-        self.sizePolicy().setControlType(QtGui.QSizePolicy.Slider)
-        self.setMinimumWidth(20)
-        self.setMaximumWidth(40) # XXX TODO make this computed
-        self.setFont(textarea.font())
-        self.setMarginWidth(0, '')
-        self.setMarginWidth(1, '')
-
-        self.SendScintilla(qsci.SCI_SETCURSOR, 2)
-        self.SendScintilla(qsci.SCI_SETCARETSTYLE, 0)
-
-        # used to set a background color for every annotating rev
-        N = 32
-        self.markers = []
-        for i in range(N):
-            marker = self.markerDefine(qsci.Background)
-            color = 0x7FFF00 + (i-N/2)*256/N*256*256 - i*256/N*256 + i*256/N
-            self.SendScintilla(qsci.SCI_MARKERSETBACK, marker, color)
-            self.markers.append(marker)
-
-        textarea.verticalScrollBar().valueChanged[int].connect(
-                self.verticalScrollBar().setValue)
-
-    def setFilectx(self, fctx):
-        ann = [f.rev() for f, __ in fctx.annotate(follow=True)]
-        self.setText('\n'.join(imap(str, ann)))
-        allrevs = list(sorted(set(ann)))
-        for i, rev in enumerate(ann):
-            idx = allrevs.index(rev)
-            self.markerAdd(i, self.markers[idx % len(self.markers)])
+from hgviewlib.qt4.widgets import SourceViewer, Annotator
 
 
-class HgQsci(qsci):
-
+class HgQsci(ActionsMixin, SourceViewer):
     def __init__(self, *args, **kwargs):
         super(HgQsci, self).__init__(*args, **kwargs)
-        super(HgQsci, self).setUtf8(True)
-        self.actions = {}
         self.createActions()
-        self.setFrameStyle(0)
-
-        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
-        self.setReadOnly(True)
-
-        self.SendScintilla(qsci.SCI_SETCARETSTYLE, 0)
-
-        # margin 1 is used for line numbers
-        self.setMarginLineNumbers(1, True)
-        self.setMarginWidth(1, '000')
-
-        self.SendScintilla(qsci.SCI_INDICSETSTYLE, 8, qsci.INDIC_ROUNDBOX)
-        self.SendScintilla(qsci.SCI_INDICSETUNDER, 8, True)
-        self.SendScintilla(qsci.SCI_INDICSETFORE, 8, 0xBBFFFF)
-        self.SendScintilla(qsci.SCI_INDICSETSTYLE, 9, qsci.INDIC_ROUNDBOX)
-        self.SendScintilla(qsci.SCI_INDICSETUNDER, 9, True)
-        self.SendScintilla(qsci.SCI_INDICSETFORE, 9, 0x58A8FF)
-
-        # hide margin 0 (markers)
-        self.SendScintilla(qsci.SCI_SETMARGINTYPEN, 0, 0)
-        self.SendScintilla(qsci.SCI_SETMARGINWIDTHN, 0, 0)
-
-        # define markers for colorize zones of diff
-        self.markerplus = self.markerDefine(qsci.Background)
-        self.SendScintilla(qsci.SCI_MARKERSETBACK, self.markerplus, 0xB0FFA0)
-        self.markerminus = self.markerDefine(qsci.Background)
-        self.SendScintilla(qsci.SCI_MARKERSETBACK, self.markerminus, 0xA0A0FF)
-        self.markertriangle = self.markerDefine(qsci.Background)
-        self.SendScintilla(qsci.SCI_MARKERSETBACK, self.markertriangle, 0xFFA0A0)
-
-    def _action_defs(self):
-        return [
-            ("diffmode", self.tr("Diff mode"), 'diffmode' ,
-             self.tr('Enable/Disable Diff mode'), None, None),
-            ("ignorews", self.tr("Ignore all space"), None ,
-             self.tr('Enable/Disable white space checking when comparing lines'), None, None),
-            ("annmode", self.tr("Annotate mode"), None,
-             self.tr('Enable/Disable Annotate mode'), None, None),
-            ("next", self.tr('Next hunk'), 'down', self.tr('Jump to the next hunk'),
-             Qt.ALT + Qt.Key_Down, None),
-            ("prev", self.tr('Prior hunk'), 'up', self.tr('Jump to the previous hunk'),
-             Qt.ALT + Qt.Key_Up, None),
-            ("show-big-file", self.tr('Display heavy file'), 'heavy',
-             self.tr('Display file Content even if it is marked as too big'
-                     '[config: maxfilesize]'),
-             None, None),
-
-        ]
 
     def createActions(self):
-        self.actions = {}
-        for name, desc, icon, tip, key, cb in self._action_defs():
-            act = QtGui.QAction(desc, self)
-            if icon:
-                act.setIcon(geticon(icon))
-            if tip:
-                act.setStatusTip(tip)
-            if key:
-                act.setShortcut(key)
-            if cb:
-                act.triggered.connect(cb)
-            self.actions[name] = act
-            self.addAction(act)
-        self.actions['diffmode'].setCheckable(True)
-        self.actions['ignorews'].setCheckable(True)
-        self.actions['annmode'].setCheckable(True)
-        self.actions['show-big-file'].setCheckable(True)
+        self.add_action(
+            "diffmode", self.tr("Diff mode"),
+            menu=self.tr("Mode"),
+            icon='diffmode' ,
+            tip=self.tr('Enable/Disable Diff mode'),
+            checked=True,
+            )
 
-    def contextMenuEvent(self, event):
-        menu = QtGui.QMenu(self)
-        for act in [None, 'diffmode', 'prev', 'next',
-                    None, 'show-big-file', None, 'ignorews']:
-            if act:
-                menu.addAction(self.actions[act])
-            else:
-                menu.addSeparator()
-        menu.exec_(event.globalPos())
+        self.add_action(
+            "ignorews", self.tr("Ignore all space"),
+            menu=self.tr("Mode"),
+            tip=self.tr("Ignore all space"),
+            checked=True,
+            )
 
-    def clear_highlights(self):
-        n = self.length()
-        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 8) # highlight
-        self.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, n)
-        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 9) # current found
-                                                            # occurrence
-        self.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, n)
+        self.add_action(
+            "show-big-file", self.tr('Display heavy file'),
+            menu=self.tr("Mode"),
+            icon='heavy',
+            tip=self.tr('Display file Content even if it is marked as too big'
+                        '[config: maxfilesize]'),
+            checked=False,
+            )
 
-    def highlight_current_search_string(self, pos, text):
-        line = self.SendScintilla(qsci.SCI_LINEFROMPOSITION, pos)
-        self.ensureLineVisible(line)
-        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 9)
-        self.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, pos)
-        self.SendScintilla(qsci.SCI_INDICATORFILLRANGE, pos, len(text))
+        self.add_action(
+            "annmode", self.tr("Annotate mode"),
+            menu=self.tr("Mode"),
+            tip=self.tr('Enable/Disable Annotate mode'),
+            checked=True,
+            )
 
-    def search_and_highlight_string(self, text):
-        data = unicode(self.text())
-        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 8)
-        pos = [data.find(text)]
-        n = len(text)
-        while pos[-1] > -1:
-            self.SendScintilla(qsci.SCI_INDICATORFILLRANGE, pos[-1], n)
-            pos.append(data.find(text, pos[-1]+1))
-        return [x for x in pos if x > -1]
+        act = self.add_action(
+            "openexternal", self.tr("Open in external application"),
+            menu=self.tr("View"),
+            tip=self.tr("Open file in an external application at the current "
+                        "revision"),
+            )
 
+        self.add_action(
+            "next", self.tr('Next hunk'),
+            menu=self.tr("Moves"),
+            icon='down',
+            tip=self.tr('Jump to the next hunk'),
+            keys=[Qt.ALT + Qt.Key_Down]
+            )
 
-class HgFileView(QtGui.QFrame):
+        self.add_action(
+            "prev", self.tr('Prior hunk'),
+            menu=self.tr("Moves"),
+            icon='up',
+            tip=self.tr('Jump to the previous hunk'),
+            keys=[Qt.ALT + Qt.Key_Up]
+            )
 
+    def toggle_openexternal(self, status=None):
+        openexternal = self.get_action('openexternal')
+        if status is None:
+            status = not openexternal.isEnabled()
+        openexternal.setEnabled(status)
+
+class HgFileView(ActionsMixin, QtGui.QFrame):
+
+    filled = pyqtSignal()
     message_logged = pyqtSignal(str, int)
     rev_for_diff_changed = pyqtSignal(int)
 
@@ -200,7 +116,7 @@ class HgFileView(QtGui.QFrame):
         self._diff = None
         self._diffs = None
         self.cfg = None
-        QtGui.QFrame.__init__(self, parent)
+        super(HgFileView, self).__init__(parent)
         framelayout = QtGui.QVBoxLayout(self)
         framelayout.setContentsMargins(0, 0, 0, 0)
         framelayout.setSpacing(0)
@@ -268,17 +184,18 @@ class HgFileView(QtGui.QFrame):
         self.timer = QtCore.QTimer()
         self.timer.setSingleShot(False)
         self.timer.timeout.connect(self.idle_fill_files)
-        self.sci.actions['diffmode'].toggled[bool].connect(self.setMode)
-        self.sci.actions['ignorews'].toggled[bool].connect(
-            lambda value: self.setUiConfig('diff', 'ignorews', value))
-        self.sci.actions['annmode'].toggled[bool].connect(self.setAnnotate)
-        self.sci.actions['prev'].triggered.connect(self.prevDiff)
-        self.sci.actions['next'].triggered.connect(self.nextDiff)
-        self.sci.actions['show-big-file'].toggled[bool].connect(self.showBigFile)
-        self.sci.actions['diffmode'].setChecked(True)
+        self.sci.set_action('diffmode', callback=self.setMode)
+        self.sci.set_action('ignorews',
+                            callback=lambda value: self.setUiConfig('diff', 'ignorews', value))
+        self.sci.set_action('annmode', callback=self.setAnnotate)
+        self.sci.set_action('prev', callback=self.prevDiff)
+        self.sci.set_action('next', callback=self.nextDiff)
+        self.sci.set_action('show-big-file', callback=self.showBigFile)
+        self.sci.set_action('openexternal', callback=self.openexternal)
+        self.sci.set_action('diffmode', checked=True)
 
     def resizeEvent(self, event):
-        QtGui.QFrame.resizeEvent(self, event)
+        super(HgFileView, self).resizeEvent(event)
         h = self.sci.horizontalScrollBar().height()
         self._spacer.setMinimumHeight(h)
         self._spacer.setMaximumHeight(h)
@@ -300,9 +217,7 @@ class HgFileView(QtGui.QFrame):
             mode = ['file', 'diff'][mode]
         assert mode in ('diff', 'file')
 
-        self.sci.actions['annmode'].setEnabled(not mode)
-        self.sci.actions['next'].setEnabled(not mode)
-        self.sci.actions['prev'].setEnabled(not mode)
+        self.sci.set_actions('annmode', 'next', 'prev', enabled=not mode)
         if mode != self._mode:
             self._mode = mode
             self.blk.setVisible(self._mode == 'file')
@@ -328,9 +243,11 @@ class HgFileView(QtGui.QFrame):
         if self._model.graph:
             is_show_big_file = self._model.graph.maxfilesize < 0
         else:
-            is_show_big_file = self.cfg.getMaxFileSize()
-        self.sci.actions['show-big-file'].setChecked(is_show_big_file)
-        self.sci.actions['ignorews'].setChecked(model.repo.ui.configbool('diff', 'ignorews'))
+            is_show_big_file = bool(self.cfg.getMaxFileSize())
+        self.sci.set_action('show-big-file', checked=is_show_big_file)
+        self.sci.set_action('ignorews',
+                            checked=model.repo.ui.configbool('diff', 'ignorews'))
+        self.sci.setFont(get_font(self.cfg))
         self.sci.clear()
 
     def setContext(self, ctx):
@@ -370,7 +287,10 @@ class HgFileView(QtGui.QFrame):
         try:
             filectx = self._ctx.filectx(self._realfilename)
 
+        except (LookupError, ManifestLookupError): # occur on deleted files
+            self.sci.toggle_openexternal(status=False)
         except HgLookupError: # occur on deleted files
+            self.sci.toggle_openexternal(status=False)
             return
         if self._mode == 'diff' and self._p_rev is not None:
             mode = self._p_rev
@@ -380,6 +300,8 @@ class HgFileView(QtGui.QFrame):
             flag, data = self._model.graph.filedata(filename, self._ctx.rev(), mode, maxfilesize=-1)
         else:
             flag, data = self._model.graph.filedata(filename, self._ctx.rev(), mode)
+        if data and data[-1] == '\n':
+            data = data[:-1]
         if flag == 'file too big':
             self.filedata_frame.hide()
             message = (('<center>'
@@ -393,25 +315,19 @@ class HgFileView(QtGui.QFrame):
             return
         else:
             self.filedata_frame.show()
-        if flag == '-':
-            return
-        if flag == '':
+        if flag == '-' or flag == '':
+            self.sci.toggle_openexternal(status=False)
             return
+        self.sci.toggle_openexternal(status=True)
 
-        lexer = get_lexer(filename, data, flag, self.cfg)
-        if flag == "+":
-            nlines = data.count('\n')
-            self.sci.setMarginWidth(1, str(nlines)+'0')
-        self.sci.setLexer(lexer)
-        self._cur_lexer = lexer
         if data not in (u'file too big', u'binary file'):
             self.filedata = data
         else:
             self.filedata = None
 
-        flag = exec_flag_changed(filectx)
-        if flag:
-            self.execflaglabel.setText(u"<b>exec mode has been <font color='red'>%s</font></b>" % flag)
+        exec_flag = exec_flag_changed(filectx)
+        if exec_flag:
+            self.execflaglabel.setText(u"<b>exec mode has been <font color='red'>%s</font></b>" % exec_flag)
             self.execflaglabel.show()
         else:
             self.execflaglabel.hide()
@@ -428,23 +344,31 @@ class HgFileView(QtGui.QFrame):
             labeltxt += u' <i>(renamed from %s)</i>' % tounicode(bfilepath(renamed[0]))
         self.filenamelabel.setText(labeltxt)
 
-        self.sci.setText(data)
+        self.sci.set_text(filename, data, flag, self.cfg)
         if self._find_text:
             self.highlightSearchString(self._find_text)
-        self.sci.actions['prev'].setEnabled(False)
+        self.sci.set_action('prev', enabled=False)
         self.updateDiffDecorations()
         if self._mode == 'file' and self._annotate:
             if filectx.rev() is None: # XXX hide also for binary files
                 self.ann.setVisible(False)
             else:
                 self.ann.setVisible(self._annotate)
-                if lexer is not None:
-                    self.ann.setFont(lexer.font(0))
-                else:
-                    self.ann.setFont(self.sci.font())
-                self.ann.setFilectx(filectx)
+                self.ann.setFont(self.sci.font())
+                self.ann.set_line_ticks([str(f.rev()) for f, __ in filectx.annotate(follow=True)])
         return True
 
+    def openexternal(self):
+        """Open the external application with the content of the selected file at
+        the selected revision"""
+        # We open the current file if the selected revision is the dirty working
+        # directory or if it is the working directory without any modification.
+        # Else we use a temporary file.
+        content_getter = lambda: self._model.graph.filedata(
+            self._filename, self._ctx.rev(), 'file', maxfilesize=-1)[1]
+        _open_in_external(self, self.cfg, self._ctx.filectx(self._filename),
+                          content_getter)
+
     def updateDiffDecorations(self):
         """
         Recompute the diff and starts the timer
@@ -491,8 +415,9 @@ class HgFileView(QtGui.QFrame):
 
     def nextDiff(self):
         notlast = self._nextDiff()
-        self.sci.actions['next'].setEnabled(self.fileMode() and notlast and self.nDiffs())
-        self.sci.actions['prev'].setEnabled(self.fileMode() and self.nDiffs())
+        self.sci.set_action('next',
+                            enabled=self.fileMode() and notlast and self.nDiffs())
+        self.sci.set_action('prev', enabled=self.fileMode() and self.nDiffs())
 
     def _prevDiff(self):
         if self._mode == 'file':
@@ -510,8 +435,9 @@ class HgFileView(QtGui.QFrame):
 
     def prevDiff(self):
         notfirst = self._prevDiff()
-        self.sci.actions['prev'].setEnabled(self.fileMode() and notfirst and self.nDiffs())
-        self.sci.actions['next'].setEnabled(self.fileMode() and self.nDiffs())
+        self.sci.set_action('prev',
+                            enabled=self.fileMode() and notfirst and self.nDiffs())
+        self.sci.set_action('next', enabled=self.fileMode() and self.nDiffs())
 
     def nextLine(self):
         x, y = self.sci.getCursorPosition()
@@ -569,7 +495,8 @@ class HgFileView(QtGui.QFrame):
             if self._diff is None or not self._diff.get_opcodes():
                 self._diff = None
                 self.timer.stop()
-                self.sci.actions['next'].setEnabled(self.fileMode() and self.nDiffs())
+                self.filled.emit()
+                self.sci.set_action('next', enabled=self.fileMode() and self.nDiffs())
                 break
 
             tag, __, __, blo, bhi = self._diff.get_opcodes().pop(0)
@@ -599,7 +526,7 @@ class HgFileView(QtGui.QFrame):
         self.blk.setUpdatesEnabled(True)
 
 
-class HgFileListView(QtGui.QTableView):
+class HgFileListView(ActionsMixin, QtGui.QTableView):
     """
     A QTableView for displaying a HgFileListModel
     """
@@ -607,7 +534,7 @@ class HgFileListView(QtGui.QTableView):
     file_selected = pyqtSignal([str, int], [str])
 
     def __init__(self, parent=None):
-        QtGui.QTableView.__init__(self, parent)
+        super(HgFileListView, self).__init__(parent)
         self.setShowGrid(False)
         self.verticalHeader().hide()
         self.verticalHeader().setDefaultSectionSize(20)
@@ -630,7 +557,7 @@ class HgFileListView(QtGui.QTableView):
         self._nav_dialogs = {}
 
     def setModel(self, model):
-        QtGui.QTableView.setModel(self, model)
+        super(HgFileListView, self).setModel(model)
         model.layoutChanged.connect(self.fileSelected)
         self.selectionModel().currentRowChanged.connect(
                 self.fileSelected)
@@ -640,11 +567,19 @@ class HgFileListView(QtGui.QTableView):
         index = self.currentIndex()
         return self.model().fileFromIndex(index)
 
+    def toggle_openexternal(self, status=None):
+        openexternal = self.get_action('openexternal')
+        if status is None:
+            status = not openexternal.isEnabled()
+        openexternal.setEnabled(status)
+
     def fileSelected(self, index=None, *args):
         if index is None:
             index = self.currentIndex()
         sel_file = self.model().fileFromIndex(index)
         from_rev = self.model().revFromIndex(index)
+        if sel_file is not None:
+            self.toggle_openexternal(self.model().fileflag(sel_file) != '-')
         # signal get unicode as input
         if from_rev is None:
             self.file_selected[str].emit(tounicode(sel_file))
@@ -666,10 +601,25 @@ class HgFileListView(QtGui.QTableView):
     def toggleFullFileList(self, *args):
         self.model().toggleFullFileList()
 
+    def openexternal(self):
+        """Open an external application with the content of the selected file at
+        the selected revision"""
+        index = self.currentIndex()
+        sel_file = self.model().fileFromIndex(index)
+        from_rev = self.model().current_ctx.rev()
+        try:
+            filectx = self.model().repo[from_rev].filectx(sel_file)
+        except (LookupError, ManifestLookupError):
+            return
+        _open_in_external(self, HgConfig(self.model().repo.ui),
+                          filectx, filectx.data)
+
     def navigate(self, filename=None):
+        from hgviewlib.qt4.hgfiledialog import FileViewer
         self._navigate(filename, FileViewer, self._nav_dialogs)
 
     def diffNavigate(self, filename=None):
+        from hgviewlib.qt4.hgfiledialog import FileDiffViewer
         self._navigate(filename, FileDiffViewer, self._diff_dialogs)
 
     def _navigate(self, filename, dlgclass, dlgdict):
@@ -689,37 +639,28 @@ class HgFileListView(QtGui.QTableView):
             dlg.raise_()
             dlg.activateWindow()
 
-    def _action_defs(self):
-        a = [("navigate", self.tr("Navigate"), None ,
-              self.tr('Navigate the revision tree of this file'), None, self.navigate),
-             ("diffnavigate", self.tr("Diff-mode navigate"), None,
-              self.tr('Navigate the revision tree of this file in diff mode'), None, self.diffNavigate),
-             ]
-        return a
-
     def createActions(self):
-        self._actions = {}
-        for name, desc, icon, tip, key, cb in self._action_defs():
-            act = QtGui.QAction(desc, self)
-            if icon:
-                act.setIcon(geticon(icon))
-            if tip:
-                act.setStatusTip(tip)
-            if key:
-                act.setShortcut(key)
-            if cb:
-                act.triggered.connect(cb)
-            self._actions[name] = act
-            self.addAction(act)
+        self.add_action(
+            'navigate', self.tr("Navigate"),
+            menu=self.tr("navigate"),
+            tip=self.tr('Navigate the revision tree of this file'),
+            callback=lambda: self.navigate(),
+            )
 
-    def contextMenuEvent(self, event):
-        menu = QtGui.QMenu(self)
-        for act in ['navigate', 'diffnavigate']:
-            if act:
-                menu.addAction(self._actions[act])
-            else:
-                menu.addSeparator()
-        menu.exec_(event.globalPos())
+        self.add_action(
+            'diffnavigate', self.tr("Diff-mode navigate"),
+            menu=self.tr("navigate"),
+            tip=self.tr('Navigate the history of this file in diff mode'),
+            callback=lambda: self.diffNavigate(),
+            )
+
+        self.add_action(
+            "openexternal", self.tr("Open in external application"),
+            menu=self.tr("View"),
+            tip=self.tr("Open file in an external application at the current "
+                        "revision"),
+            callback=self.openexternal,
+        )
 
     def resizeEvent(self, event):
         vp_width = self.viewport().width()
@@ -741,3 +682,43 @@ class HgFileListView(QtGui.QTableView):
     def prevFile(self):
         row = self.currentIndex().row()
         self.setCurrentIndex(self.model().index(max(row - 1, 0), 0))
+
+
+def _open_in_external(parent, cfg, filectx, content_getter):
+    """Open an external application on the file at the context ``filectx``.
+
+    If the selected revision is the working directory then the original file
+    opened else a temporary file is created.
+    """
+    if filectx.rev() is None: # dirty wd is selected
+        return _open_originalfile_in_external(parent, filectx)
+    if ((not filectx._repo[None].dirty()) and                    # the wd is clean and
+        (filectx.changectx() in filectx._repo[None].parents())): # a wd parent is selected
+        return _open_originalfile_in_external(parent, filectx)
+    content = content_getter()
+    suffix = '_%s-%s-%s' % (filectx.rev(), str(filectx.changectx()),
+                            os.path.basename(filectx.path()))
+    _open_tempfile_in_external(parent, content, suffix=suffix)
+
+
+def _open_originalfile_in_external(parent, filectx):
+    """Open an external application on the original file from filectx"""
+    filepath = os.path.join(
+        os.path.abspath(filectx._repo.root),
+        filectx.path()
+        )
+    return QtGui.QDesktopServices.openUrl(QtCore.QUrl(filepath))
+
+
+def _open_tempfile_in_external(parent, content, suffix=None):
+    """Open an external application with the given ``content`` in a temporary
+    file.
+
+    ``suffix`` is the suffix for the temporary file name.
+    ``parent`` is a Qt component that gets a reference to the editor process.
+    """
+    fid, filepath = tempfile.mkstemp(suffix=suffix)
+    os.close(fid)
+    with open(filepath, 'w') as fid:
+        fid.write(content)
+    return QtGui.QDesktopServices.openUrl(QtCore.QUrl(filepath))
